# -*- coding: utf-8 -*-
"""
JSON encoder/decoder adapted for use with Google App Engine NDB.

Usage:

  import ndb_json

  # Serialize an ndb.Query into an array of JSON objects.
  query = models.MyModel.query()
  query_json = ndb_json.dumps(query)

  # Convert into a list of Python dictionaries.
  query_dicts = ndb_json.loads(query_json)

  # Serialize an ndb.Model instance into a JSON object.
  entity = query.get()
  entity_json = ndb_json.dumps(entity)

  # Convert into a Python dictionary.
  entity_dict = ndb_json.loads(entity_json)


Dependencies:

  - dateutil: https://pypi.python.org/pypi/python-dateutil
"""

__author__ = 'Eric Higgins'
__copyright__ = 'Copyright 2013-2016, Eric Higgins'
__email__ = 'erichiggins@gmail.com'


import base64
import datetime
import json
import time
import types

import dateutil.parser
from google.appengine.ext import ndb


__all__ = (
    'dump',
    'dumps',
    'loads',
    'NdbDecoder',
    'NdbEncoder',
)


def encode_model(obj):
  """Encode objects like ndb.Model which have a `.to_dict()` method."""
  obj_dict = obj.to_dict()
  for key, val in obj_dict.iteritems():
    if isinstance(val, types.StringType):
      try:
        unicode(val)
      except UnicodeDecodeError:
        # Encode binary strings (blobs) to base64.
        obj_dict[key] = base64.b64encode(val)
  return obj_dict


def encode_generator(obj):
  """Encode generator-like objects, such as ndb.Query."""
  return list(obj)


def encode_key_as_entity(obj):
  """Get the Entity from the ndb.Key for further encoding."""
  # NOTE(erichiggins): Potentially poor performance for Models w/ many KeyProperty properties.
  # NOTE(ronufryk): Potentially can cause circular references and "RuntimeError: maximum recursion depth exceeded"
  return obj.get_async()


# Alias for backward-compatibility
encode_key = encode_key_as_entity


def encode_key_as_pair(obj):
  """Get the ndb.Key as a tuple of (kind, id) pairs."""
  return obj.pairs()


def encode_key_as_urlsafe(obj):
  """Get the ndb.Key as URL-safe base64-encoded string."""
  return obj.urlsafe()


def encode_future(obj):
  """Encode an ndb.Future instance."""
  return obj.get_result()


def encode_datetime(obj):
  """Encode a datetime.datetime or datetime.date object as an ISO 8601 format string."""
  # Reformat the date slightly for better JS compatibility.
  # Offset-naive dates need 'Z' appended for JS.
  # datetime.date objects don't have or need tzinfo, so don't append 'Z'.
  zone = '' if getattr(obj, 'tzinfo', True) else 'Z'
  return obj.isoformat() + zone


def encode_complex(obj):
  """Convert a complex number object into a list containing the real and imaginary values."""
  return [obj.real, obj.imag]


def encode_basevalue(obj):
  """Retrieve the actual value from a ndb.model._BaseValue.

  This is a convenience function to assist with the following issue:
  https://code.google.com/p/appengine-ndb-experiment/issues/detail?id=208
  """
  return obj.b_val


NDB_TYPE_ENCODING = {
  ndb.MetaModel: encode_model,
  ndb.Query: encode_generator,
  ndb.QueryIterator: encode_generator,
  ndb.Key: encode_key_as_entity,
  ndb.Future: encode_future,
  datetime.date: encode_datetime,
  datetime.datetime: encode_datetime,
  time.struct_time: encode_generator,
  types.ComplexType: encode_complex,
  ndb.model._BaseValue: encode_basevalue,
}

# Sort the types so any iteration is in a deterministic order
NDB_TYPES = sorted(NDB_TYPE_ENCODING.keys(), key=lambda t: t.__name__)


class NdbDecoder(json.JSONDecoder):
  """Extend the JSON decoder to add support for datetime objects."""

  def __init__(self, **kwargs):
    """Override the default __init__ in order to specify our own parameters."""
    json.JSONDecoder.__init__(self, object_hook=self.object_hook_handler, **kwargs)

  def object_hook_handler(self, val):
    """Handles decoding of nested date strings."""
    return {k: self.decode_date(v) for k, v in val.iteritems()}

  def decode_date(self, val):
    """Tries to decode strings that look like dates into datetime objects."""
    if isinstance(val, basestring) and val.count('-') == 2 and len(val) > 9:
      try:
        dt = dateutil.parser.parse(val)
        # Check for UTC.
        if val.endswith(('+00:00', '-00:00', 'Z')):
          # Then remove tzinfo for gae, which is offset-naive.
          dt = dt.replace(tzinfo=None)
        return dt
      except (TypeError, ValueError):
        pass
    return val

  def decode(self, val):
    """Override of the default decode method that also uses decode_date."""
    # First try the date decoder.
    new_val = self.decode_date(val)
    if val != new_val:
      return new_val
    # Fall back to the default decoder.
    return json.JSONDecoder.decode(self, val)


class NdbEncoder(json.JSONEncoder):
  """Extend the JSON encoder to add support for NDB Models."""


  def __init__(self, **kwargs):
    self._ndb_type_encoding = NDB_TYPE_ENCODING.copy()

    keys_as_entities = kwargs.pop('ndb_keys_as_entities', False)
    keys_as_pairs = kwargs.pop('ndb_keys_as_pairs', False)
    keys_as_urlsafe = kwargs.pop('ndb_keys_as_urlsafe', False)

    # Validate that only one of three flags is True
    if ((keys_as_entities and keys_as_pairs)
        or (keys_as_entities and keys_as_urlsafe)
        or (keys_as_pairs and keys_as_urlsafe)):
      raise ValueError('Only one of arguments ndb_keys_as_entities, ndb_keys_as_pairs, ndb_keys_as_urlsafe can be True')

    if keys_as_pairs:
      self._ndb_type_encoding[ndb.Key] = encode_key_as_pair
    elif keys_as_urlsafe:
      self._ndb_type_encoding[ndb.Key] = encode_key_as_urlsafe
    else:
      self._ndb_type_encoding[ndb.Key] = encode_key_as_entity


    json.JSONEncoder.__init__(self, **kwargs)

  def default(self, obj):
    """Overriding the default JSONEncoder.default for NDB support."""
    obj_type = type(obj)
    # NDB Models return a repr to calls from type().
    if obj_type not in self._ndb_type_encoding:
      if hasattr(obj, '__metaclass__'):
        obj_type = obj.__metaclass__
      else:
        # Try to encode subclasses of types
        for ndb_type in NDB_TYPES:
          if isinstance(obj, ndb_type):
            obj_type = ndb_type
            break

    fn = self._ndb_type_encoding.get(obj_type)

    if fn:
      return fn(obj)

    return json.JSONEncoder.default(self, obj)


def dumps(ndb_model, **kwargs):
  """Custom json dumps using the custom encoder above."""
  return NdbEncoder(**kwargs).encode(ndb_model)


def dump(ndb_model, fp, **kwargs):
  """Custom json dump using the custom encoder above."""
  for chunk in NdbEncoder(**kwargs).iterencode(ndb_model):
    fp.write(chunk)


def loads(json_str, **kwargs):
  """Custom json loads function that converts datetime strings."""
  return NdbDecoder(**kwargs).decode(json_str)



# -*- coding: utf-8 -*-

"""
test_ndb_json
----------------------------------

Tests for `gaek` module.
"""

import datetime
import json
import mock
import unittest
import cStringIO

from google.appengine.ext import ndb
from nose import tools

from gaek import ndb_json


__all__ = [
  'TestNdbJson',
]


class TestNdbJson(unittest.TestCase):

    def setUp(self):
      self.ndb_mock = mock.Mock()
      self.ndb_mock.__metaclass__ = ndb.Key
      self.ndb_mock.urlsafe = mock.Mock(return_value='urlsafe')
      self.ndb_mock.pairs = mock.Mock(return_value='pairs')
      self.ndb_mock.get_async = mock.Mock(return_value='get_async')
      pass

    def tearDown(self):
        pass

    def test_dumps_with_ndb_values(self):
        """Assert functional with NDB datastore values."""
        pass

    def test_dumps_with_naive_values(self):
        """Assert functional parity with `json.dump` for naive values."""
        assert json.dumps({'bool true': True}) == ndb_json.dumps({'bool true': True})
        assert json.dumps({'bool false': False}) == ndb_json.dumps({'bool false': False})
        assert json.dumps({'empty str': ''}) == ndb_json.dumps({'empty str': ''})
        assert json.dumps({'str': 'hello world'}) == ndb_json.dumps({'str': 'hello world'})
        assert json.dumps({'unicode': u'(╯°□°)╯︵ ┻━┻'}) == ndb_json.dumps({'unicode': u'(╯°□°)╯︵ ┻━┻'})
        assert json.dumps({'int': 12345}) == ndb_json.dumps({'int': 12345})
        assert json.dumps({'float': 1.2345}) == ndb_json.dumps({'float': 1.2345})
        assert json.dumps({'str int': '12345'}) == ndb_json.dumps({'str int': '12345'})
        assert json.dumps({'str float': '1.2345'}) == ndb_json.dumps({'str float': '1.2345'})

    def test_dump_with_naive_values(self):
        """Assert functional parity with `json.dump`, for use with file-like objects."""
        json_fp = cStringIO.StringIO()
        ndb_json_fp = cStringIO.StringIO()

        payload = {
            'unicode': u'(╯°□°)╯︵ ┻━┻',
        }

        json.dump(payload, json_fp)
        ndb_json.dump(payload, ndb_json_fp)

        assert json_fp.getvalue() == ndb_json_fp.getvalue()
        json_fp.close()
        ndb_json_fp.close()

    def test_dumps_with_subclassed_type(self):
        """ Assert that a subclass of a supported type will encode as JSON properly """
        class MyDateTime(datetime.datetime):
            pass

        subclass_parsed = ndb_json.dumps({'a datetime': MyDateTime(2015, 10, 1)})
        original_parsed = ndb_json.dumps({'a datetime': datetime.datetime(2015, 10, 1)})
        assert subclass_parsed == original_parsed

    def test_loads_with_naive_values(self):
        payload_str = json.dumps({
            'bool true': True,
            'bool false': False,
            'empty str': '',
            'str': 'hello world',
            'unicode': u'(╯°□°)╯︵ ┻━┻',
            'int': 12345,
            'float': 1.2345,
            'str int': '12345',
            'str float': '1.2345',
        })

        parsed = ndb_json.loads(payload_str)

        assert True == parsed['bool true']
        assert False == parsed['bool false']
        assert '' == parsed['empty str']
        assert 'hello world' == parsed['str']
        assert u'(╯°□°)╯︵ ┻━┻' == parsed['unicode']
        assert 12345 == parsed['int']
        assert 1.2345 == parsed['float']
        assert '12345' == parsed['str int']
        assert '1.2345' == parsed['str float']

    def test_loads_with_date_and_time_values(self):
        """Assert that date/time-like strings are parsed properly."""
        assert datetime.datetime(2015, 1, 1) == ndb_json.loads(datetime.date(2015, 1, 1).isoformat())

        payload_str = json.dumps({
            'date': datetime.date(2015, 1, 1).isoformat(),
            'datetime': datetime.datetime(2015, 1, 1, 12, 0, 0, 0).isoformat(),
            'time': '12:30',
            'non-date': '12-15',
            'double-hyphen non-date': '12-15-0',
        })

        parsed = ndb_json.loads(payload_str)

        # TODO(erichiggins): It may be better for dates to be parsed as instances of `datetime.date`.
        assert datetime.datetime(2015, 1, 1) == parsed['date']
        assert datetime.datetime(2015, 1, 1, 12, 0, 0, 0) == parsed['datetime']
        assert '12:30' == parsed['time']
        assert '12-15' == parsed['non-date']
        assert '12-15-0' == parsed['double-hyphen non-date']

    def test_loads_with_nested_datetime(self):
        """Assert the object hooks work as intended."""
        payload_str = json.dumps({
          'nested': {'datetime': datetime.datetime(2016, 1, 1, 12).isoformat()},
        })
        parsed = ndb_json.loads(payload_str)
        assert datetime.datetime(2016, 1, 1, 12, 0, 0, 0) == parsed['nested']['datetime']

    def test_invalid_arguments(self):
      self.assertRaises(ValueError,
                        ndb_json.NdbEncoder,
                        ndb_keys_as_entities=True,
                        ndb_keys_as_pairs=True,
                        ndb_keys_as_urlsafe=True)
      self.assertRaises(ValueError,
                        ndb_json.NdbEncoder,
                        ndb_keys_as_entities=False,
                        ndb_keys_as_pairs=True,
                        ndb_keys_as_urlsafe=True)
      self.assertRaises(ValueError,
                        ndb_json.NdbEncoder,
                        ndb_keys_as_entities=True,
                        ndb_keys_as_pairs=False,
                        ndb_keys_as_urlsafe=True)
      self.assertRaises(ValueError,
                        ndb_json.NdbEncoder,
                        ndb_keys_as_entities=True,
                        ndb_keys_as_pairs=True,
                        ndb_keys_as_urlsafe=False)

    def test_encode_key(self):
      some_obj = mock.Mock()
      ndb_json.encode_key(some_obj)
      self.assertEqual(1, some_obj.get_async.call_count)

    def test_encode_key_as_entity(self):
      some_obj = mock.Mock()
      ndb_json.encode_key_as_entity(some_obj)
      self.assertEqual(1, some_obj.get_async.call_count)

    def test_encode_key_as_pair(self):
      some_obj = mock.Mock()
      ndb_json.encode_key_as_pair(some_obj)
      self.assertEqual(1, some_obj.pairs.call_count)

    def test_encode_key_as_urlsafe(self):
      some_obj = mock.Mock()
      ndb_json.encode_key_as_urlsafe(some_obj)
      self.assertEqual(1, some_obj.urlsafe.call_count)

    def test_dumps__no_option_specified(self):
      obj = {
        "number": 1,
        "string": "is here",
        "key": self.ndb_mock
      }

      dump = ndb_json.dumps(obj, sort_keys=True)
      self.assertEqual('{"key": "get_async", "number": 1, "string": "is here"}', dump)

    def test_dumps__ndb_keys_as_entities(self):

      obj = {
        "number": 1,
        "string": "is here",
        "key": self.ndb_mock
      }

      dump = ndb_json.dumps(obj, sort_keys=True, ndb_keys_as_entities=True)
      self.assertEqual('{"key": "get_async", "number": 1, "string": "is here"}', dump)

    def test_dumps__ndb_keys_as_pairs(self):

      obj = {
        "number": 1,
        "string": "is here",
        "key": self.ndb_mock
      }

      dump = ndb_json.dumps(obj, sort_keys=True, ndb_keys_as_pairs=True)
      self.assertEqual('{"key": "pairs", "number": 1, "string": "is here"}', dump)

    def test_dumps__ndb_keys_as_urlsafe(self):

      obj = {
        "number": 1,
        "string": "is here",
        "key": self.ndb_mock
      }

      dump = ndb_json.dumps(obj, sort_keys=True, ndb_keys_as_urlsafe=True)
      self.assertEqual('{"key": "urlsafe", "number": 1, "string": "is here"}', dump)

    def test_loads_with_primitive_values(self):
        """Assert that primitive values are parsed properly."""
        test_cases = ['null', '2', 'Infinity', '1.2345']
        for s in test_cases:
            tools.eq_(json.loads(s), ndb_json.loads(s))

    def test_loads_with_other_collections(self):
        """Assert that array/list values are parsed properly."""
        test_cases = ['[1,2,3,4]', '[[0]]', '[[[{}]]]']
        for s in test_cases:
            tools.eq_(json.loads(s), ndb_json.loads(s))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Environment discovery and helper functions for Google App Engine.
Some methods from the following modules have been made available for convenience:

* google.appengine.api.app_identity
  https://cloud.google.com/appengine/docs/python/appidentity/
* google.appengine.api.modules
  https://cloud.google.com/appengine/docs/python/modules/
* google.appengine.api.namespace_manager
  https://cloud.google.com/appengine/docs/python/multitenancy/

Example:

  import gaek.environ

  # Only send emails in production.
  if gaek.environ.is_production():
    mail.send(*args, **kwargs)
  
"""
 
__author__ = 'Eric Higgins'
__copyright__ = 'Copyright 2015-2016, Eric Higgins'
__email__ = 'erichiggins@gmail.com'


import os

from google.appengine.api import app_identity
from google.appengine.api import modules
from google.appengine.api import namespace_manager


__all__ = (
    # App Identity functions.
    'get_application_id',
    'get_default_version_hostname',
    'get_service_account_name',
    # Module functions.
    'get_current_instance_id',
    'get_current_module_name',
    'get_current_version_name',
    'get_default_version',
    'get_hostname',
    'get_modules',
    'get_versions',
    # Namespace functions.
    'get_namespace',
    'google_apps_namespace',
    # Helper functions.
    'get_dot_target_name',
    'get_dot_target_name_safe',
    'get_environ_dict',
    'is_host_google',
    'is_development',
    'is_staging',
    'is_staging_safe',
    'is_production',
    'is_production_safe',
    'is_default_version',
    'is_default_version_safe',
    'get_current_module_name_safe',
    'get_current_version_name_safe'
)


_UNDEFINED = '_UNDEFINED_'


# App Identity functions.
get_application_id = app_identity.get_application_id
get_default_version_hostname = app_identity.get_default_version_hostname
get_service_account_name = app_identity.get_service_account_name


# Module functions.
get_current_instance_id = modules.get_current_instance_id
get_current_module_name = modules.get_current_module_name
get_current_version_name = modules.get_current_version_name
get_default_version = modules.get_default_version
get_hostname = modules.get_hostname
get_modules = modules.get_modules
get_versions = modules.get_versions


# Namespace functions.
get_namespace = namespace_manager.get_namespace
google_apps_namespace = namespace_manager.google_apps_namespace


# Helper functions.
def get_current_version_name_safe():
  """Returns the current version of the app, or None if there is no current version found."""
  try:
    return modules.get_current_version_name()
  except KeyError:
    return None


def get_current_module_name_safe():
  """Returns the current module of the app, or None if there is no current module found.."""
  try:
    return modules.get_current_module_name()
  except KeyError:
    return None


def is_host_google():
  """True if the app is being hosted from Google App Engine servers."""
  return os.environ.get('SERVER_SOFTWARE', '').startswith('Google') or get_hostname().endswith('.appspot.com')


def is_default_version(version=None):
  """True if the current or specified app version is the default."""
  version = version or get_current_version_name()
  return version == get_default_version()


def is_default_version_safe(version=None):
  """
  True if the current or specified app version is the default.
  Returns False when there is no version found.
  """
  version = version or get_current_version_name_safe()
  return version == get_default_version()


def is_development():
  """True if the dev_appserver is running (localhost or local development server)."""
  return os.environ.get('SERVER_SOFTWARE', '').startswith('Development')


def is_staging(version=None):
  """True if the app is hosted by Google (appspot.com) but the version is not the default."""
  return is_host_google() and not is_default_version(version)


def is_staging_safe(version=None):
  """True if the app is hosted by Google (appspot.com) but the version is not the default."""
  is_default_version = is_default_version_safe()
  if is_default_version is None:
    return False
  return is_host_google() and not is_default_version


def is_production(version=None):
  """True if the app is being hosted by Google and the default version."""
  return is_host_google() and is_default_version(version)


def is_production_safe(version=None):
  """True if the app is being hosted by Google and the default version."""
  is_default_version = is_default_version_safe(version)
  if is_default_version is None:
    return False
  return is_host_google() and is_default_version


def get_dot_target_name(version=None, module=None):
  """Returns the current version/module in -dot- notation which is used by `target:` parameters."""
  version = version or get_current_version_name()
  module = module or get_current_module_name()
  return '-dot-'.join((version, module))


def get_dot_target_name_safe(version=None, module=None):
  """
  Returns the current version/module in -dot- notation which is used by `target:` parameters.
  If there is no current version or module then None is returned.
  """
  version = version or get_current_version_name_safe()
  module = module or get_current_module_name_safe()
  if version and module:
    return '-dot-'.join((version, module))
  return None


def _get_os_environ_dict(keys):
  """Return a dictionary of key/values from os.environ."""
  return {k: os.environ.get(k, _UNDEFINED) for k in keys}


def _get_app_identity_dict(keys):
  """Return a dictionary of key/values from the app_identity module functions."""
  return {k: getattr(app_identity, k)() for k in keys}


def _get_modules_dict(keys):
  """Return a dictionary of key/values from the modules module functions."""
  return {k: getattr(modules, k)() for k in keys}


def _get_namespace_manager_dict(keys):
  """Return a dictionary of key/values from the namespace_manager module functions."""
  return {k: getattr(namespace_manager, k)() for k in keys}


def get_environ_dict():
  """Return a dictionary of all environment keys/values."""
  return {
      'os.environ': _get_os_environ_dict((
          'AUTH_DOMAIN',
          'CURRENT_CONFIGURATION_VERSION',
          'CURRENT_MODULE_ID',
          'CURRENT_VERSION_ID',
          'DEFAULT_VERSION_HOSTNAME',
          'FEDERATED_IDENTITY',
          'FEDERATED_PROVIDER',
          'GAE_LOCAL_VM_RUNTIME',
          'HTTP_HOST',
          'HTTP_PROXY',
          'HTTP_X_APPENGINE_HTTPS',
          'HTTP_X_APPENGINE_QUEUENAME',
          'HTTP_X_ORIGINAL_HOST',
          'HTTP_X_ORIGINAL_SCHEME',
          'SERVER_NAME',
          'SERVER_PORT',
          'SERVER_SOFTWARE',
          'USER_IS_ADMIN',
      )),
      'app_identity': _get_app_identity_dict((
          'get_service_account_name',
          'get_application_id',
          'get_default_version_hostname',
      )),
      'modules': _get_modules_dict((
          'get_current_module_name',
          'get_current_version_name',
          'get_current_instance_id',
          'get_modules',
          'get_versions',
          'get_default_version',
          'get_hostname',
      )),
      'namespace_manager': _get_namespace_manager_dict((
          'get_namespace',
          'google_apps_namespace',
      )),
  }



# -*- coding: utf-8 -*-

"""
test_environ
----------------------------------

Tests for `environ` module.
"""

import mock
import os
import unittest

from google.appengine.api import app_identity
from google.appengine.api import modules
from google.appengine.api import namespace_manager
from google.appengine.ext import testbed

from gaek import environ


__all__ = [
  'TestEnviron',
]


class TestEnviron(unittest.TestCase):

    def setUp(self):
        # Setups app engine test bed.
        # http://code.google.com/appengine/docs/python/tools/localunittesting.html
        self.testbed = testbed.Testbed()
        self.testbed.activate()
        # Declare which service stubs you want to use.
        self.testbed.init_app_identity_stub()
        self.testbed.init_modules_stub()

    def tearDown(self):
        self.testbed.deactivate()

    def test_app_identity_functions(self):
        assert app_identity.get_application_id == environ.get_application_id
        assert app_identity.get_default_version_hostname == environ.get_default_version_hostname
        assert app_identity.get_service_account_name == environ.get_service_account_name

    def test_modules_functions(self):
        assert modules.get_current_instance_id == environ.get_current_instance_id
        assert modules.get_current_module_name == environ.get_current_module_name
        assert modules.get_current_version_name == environ.get_current_version_name
        assert modules.get_default_version == environ.get_default_version
        assert modules.get_hostname == environ.get_hostname
        assert modules.get_modules == environ.get_modules
        assert modules.get_versions == environ.get_versions

    def test_namespace_functions(self):
        assert namespace_manager.get_namespace == environ.get_namespace
        assert namespace_manager.google_apps_namespace == environ.google_apps_namespace

    def test_get_environ_dict(self):
        # TODO(eric): This one is a bit hefty.
        pass

    def test_get_dot_target_name(self):
        val = environ.get_dot_target_name()
        assert val == 'testbed-version-dot-default', repr(val)

    def test_get_dot_target_name_safe(self):
        with mock.patch('gaek.environ.get_current_version_name_safe', return_value=None):
            val = environ.get_dot_target_name_safe()
            assert val is None, val
        with mock.patch('gaek.environ.get_current_module_name_safe', return_value=None):
            val = environ.get_dot_target_name_safe()
            assert val is None, val
        val = environ.get_dot_target_name_safe()
        assert val == 'testbed-version-dot-default', repr(val)

    def test_is_host_google(self):
        val = environ.is_host_google()
        assert val == False, repr(val)

    def test_is_development(self):
        val = environ.is_development()
        assert val == True, repr(val)

    def test_is_staging(self):
        val = environ.is_staging()
        assert val == False, repr(val)

    def test_is_staging_safe(self):
        with mock.patch('gaek.environ.get_current_version_name_safe', return_value=None):
            val = environ.is_staging_safe()
            assert val is False, repr(val)
        val = environ.is_staging_safe()
        assert val == False, repr(val)

    def test_is_production(self):
        val = environ.is_production()
        assert val == False, repr(val)

    def test_is_production_safe(self):
        with mock.patch('gaek.environ.get_current_version_name_safe', return_value=None):
            val = environ.is_production_safe()
            assert val is False, repr(val)
        val = environ.is_production_safe()
        assert val == False, repr(val)

    def test_is_default_version(self):
        val = environ.is_default_version()
        assert val == False, repr(val)

    def test_is_default_version_safe(self):
        with mock.patch('gaek.environ.get_current_version_name_safe', return_value=None):
            val = environ.is_default_version_safe()
        assert val == False, repr(val)

    def test_get_current_version_name_safe(self):
        # The version is stored in an environment variable 'CURRENT_VERSION_ID'.
        #  If that variable isn't present then an error will be raised unless we catch it.
        saved_version = os.environ.pop('CURRENT_VERSION_ID', None)

        version = 'v1'
        try:
            version = environ.get_current_version_name_safe()
        except Exception as err:
            self.fail('Unexpected exception when getting current version: {}'.format(
                err.message))
        assert version is None

        os.environ['CURRENT_VERSION_ID'] = saved_version

        # Now the environment variable is back.
        version = environ.get_current_version_name_safe()
        assert 'testbed-version' == version, version

    def test_get_current_module_name_safe(self):
        """
        Test that environ.get_current_module_name returns None when there is no
        current module, rather than raising an error.
        """
        # The current module is stored in an environment variable 'CURRENT_MODULE_ID'.
        saved_module_name = os.environ.pop('CURRENT_MODULE_ID', None)

        current_module = 'v1-app'
        try:
            current_module = environ.get_current_module_name_safe()
        except Exception as err:
            self.fail('Unexpected exception when getting current module: {}'.format(
                err.message))
        assert current_module is None

        os.environ['CURRENT_MODULE_ID'] = saved_module_name
        # Now the environment variable is back.
        current_module = environ.get_current_module_name_safe()
        assert 'default' == current_module, current_module


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup
from setuptools import find_packages

# https://bugs.python.org/issue15881
try:
  import multiprocessing
except ImportError:
  pass


def readme():
  with open('README.md') as fp:
    return fp.read()


def version():
  with open('VERSION') as fp:
    return fp.readline().rstrip('\n')


def requirements():
  with open('requirements.txt') as fp:
    return fp.read()


def test_requirements():
  with open('requirements_test.txt') as fp:
    return fp.read()


setup(
    name='gaek',
    version=version(),
    description="A collection of useful tools for Python apps running on Google App Engine.",
    long_description=readme(),
    long_description_content_type='text/markdown',
    author="Eric Higgins",
    author_email='erichiggins@gmail.com',
    url='https://github.com/erichiggins/gaek',
    license="BSD",
    packages=find_packages(exclude=['tests', 'google']),
    include_package_data=True,
    install_requires=requirements(),
    test_suite='tests',
    tests_require=test_requirements(),
    zip_safe=False)
